// vincia03-root.cc is a part of the VINCIA plugin to PYTHIA 8
// This is a simple test program to study light-flavor fragmentation 
// at LEP (compared to L3 udsc measurements), using VINCIA's ROOT-based 
// runtime display. 
// Uses VINCIA for showering and matching and PYTHIA 8 for all else
// Copyright (c) 2013 Peter Skands

// PYTHIA and VINCIA include files
#include "Vincia.h"
// ROOT: include files for linking to ROOT, uncomment as necessary
#include "VinciaRoot.h"
#include "PythiaRoot.h"

// Use Pythia8 and Vincia namespaces, for convenience.
using namespace Pythia8; 
using namespace Vincia; 

// Global on/off switches for optional additional plots:
const bool OPTshowPT = false;
const bool OPTshowParticleSpectra = false;

// Main Program
int main() {

  //************************************************************************
  // PYTHIA and VINCIA objects
  //************************************************************************

  // Create Pythia and Vincia objects. 
  Pythia pythia; 
  VinciaPlugin vincia(&pythia,"vincia03.cmnd");

  //************************************************************************
  // Define some useful pointers and shorthands for later on
  //************************************************************************

  // Shorthand for event record
  Event* event = &pythia.event;

  // Some useful pointers
  Settings* settingsPtr   = vincia.settingsPtr;        // to settings database  

  // Extract settings to be used in the main program.  
  double eCM              = settingsPtr->parm("Beams:eCM");
  int    nEvent           = settingsPtr->mode("Main:numberOfEvents");  

  // Vincia settings
  bool   vinciaOn         = settingsPtr->flag("Vincia");
  int    verbose          = settingsPtr->mode("Vincia:verbose");

  // Matching settings
  int    matchingLO       = settingsPtr->mode("Vincia:matchingLO");
  bool   matching         = matchingLO >= 1;

  // hadronization settings
  bool   hadronLevel      = settingsPtr->flag("HadronLevel:all");

  //************************************************************************
  // VinciaRoot
  // Create VinciaRoot object and set plot labels, legends, etc
  // Also define names of files to save and read histograms to/from
  //************************************************************************

  // Create VinciaRoot object and set default behavior to logarithmic y axes
  VinciaRoot vroot; 
  vroot.setLogY();
  // Set range on ratio plots (default is [0.5,1.5])
  vroot.setRatioRange(0.5,1.5);

  // Tell VinciaRoot how verbose you want it to be
  vroot.setVerbose(verbose);
  
  // Set first update and save event
  double iSave   = 1000;

  // Set MC names and versions
  string pyVer = num2str(settingsPtr->parm("Pythia:versionNumber"),5);  
  string vcVer = num2str(settingsPtr->parm("Vincia:versionNumber"),5);  
  string mcVersions = "Vincia " + vcVer 
    + (matching ? " + MadGraph 4.4.26" : "") + " + Pythia "+pyVer;

  // Examples of how to set plot information and define read/save file names
  string mcLabel;
  // Example 1: PYTHIA runs
  if (!vinciaOn) {
    mcLabel    = "Pythia";
    mcVersions = "Pythia "+pyVer;
  }
  // Example 2: VINCIA runs
  else {
    mcLabel    = "Vincia";
  }
  // Example 3: parton-level results (hadronization OFF)
  if (!hadronLevel) mcLabel += " partons";
  
  // Save results of current run in data files with the following prefix
  string mcFileSave = "vincia03-last";

  // Compare current run to files with the following prefix(es)
  vector<string> mcFileRead;
  mcFileRead.push_back("validation/pythia-def/vincia03");
  mcFileRead.push_back("validation/vincia-def/vincia03");

  // Set VinciaRoot plot labels, legends, etc
  vroot.setLegend(mcLabel);
  vroot.setMCnames(mcVersions);

  //************************************************************************
  // Initialize PYTHIA (automatically initializes VINCIA) 
  //************************************************************************

  // Initialize
  pythia.init();

  //************************************************************************
  // Book Histograms
  //************************************************************************

  // Vincia uncertainty bands: number of variations expected 
  int nVariations = max(1,vincia.nVariations());

  // Default plot sizes
  double plotSizeX = 360;
  double plotSizeY = 420;

  // Histogram bin and range variables
  int nBins;
  double xLo, xHi;

  // Book alphaS histogram
  nBins = 200;
  xLo   = 0.;
  xHi   = 100.;
  TH1D *alphaS=new TH1D("alphaS","#alpha_{s}(Q)",nBins,xLo,xHi);
  TH1D *alphaS1=new TH1D("alphaS1","#alpha_{s}(Q)",nBins,xLo,xHi);
  // Set properties 
  alphaS->SetMinimum(0.0);
  alphaS->SetMaximum(0.5);
  alphaS->GetXaxis()->SetTitle("Q/GeV");
  cout<<"Creating alphaS histogram"<<endl;
  vroot.createDisplay("alphaS",alphaS,320,400);  
  vroot.addTH1DPtr("alphaS",alphaS1);
  vroot.setNormalize("alphaS",false);
  vector<string> alphaSlegend;
  alphaSlegend.push_back(string("#alpha_{s}"));
  alphaSlegend.push_back(string("#alpha_{s}^{(1-Loop)}"));
  vroot.setLegend("alphaS",alphaSlegend);

  // Fill histogram
  if (vinciaOn) {
    for ( int iBin=0; iBin<nBins; ++iBin ) {
      double Q = xLo + (iBin+0.5)*(xHi-xLo)/nBins;
      double as   = vincia.shower.alphaSptr->alphaS(pow2(Q));
      double as1  = vincia.shower.alphaSptr->alphaS1Ord(pow2(Q));
      if (as < alphaS->GetMaximum()) {
        alphaS->Fill(Q,as);
        alphaS1->Fill(Q,as1);
      } else {
        alphaS->Fill(Q,alphaS->GetMaximum());             
      }
      alphaS->SetBinError(iBin+1,0.);	
      alphaS1->SetBinError(iBin+1,0.);	
    }
    vroot.update("alphaS");
  }

  // nPartons
  TH1D *HnPartons=new TH1D("nPartons","N_{Partons}(Q_{had})", 50, -0.5, 49.5);
  HnPartons->SetMinimum(0.0005);
  HnPartons->SetMaximum(0.5);
  vroot.createDisplay("nPartons",HnPartons,320,320);
  vroot.setLogY(string("nPartons"),true);

  // Quark Pairs
  nBins = 10;
  xLo   = -0.5;
  xHi   = 9.5;
  vector<TH1D*> VnQuarks;
  VnQuarks.push_back(new TH1D("nQuarks","N_{Quarks}(Q_{had})",nBins,xLo,xHi));
  VnQuarks[0]->GetXaxis()->SetTitle("N_{Q}");
  VnQuarks[0]->GetYaxis()->SetTitle("P(N_{Q})");
  VnQuarks[0]->SetMinimum(0.00005);
  VnQuarks[0]->SetMaximum(5.);
  for (int iVar=1; iVar<nVariations; iVar++) 
    VnQuarks.push_back(new TH1D(string("nQuarks"+num2str(iVar,0)).c_str(),
			       " ",nBins,xLo,xHi));
  //vroot.createDisplay("nQuarks",VnQuarks,320,360);
  //vroot.setLogY("nQuarks");

  // Mesons
  nBins = 10;
  xLo   = -0.5;
  xHi   =  9.5;
  vector<TH1D*> VMesons;
  VMesons.push_back(new TH1D("Mesons","Meson Fractions",10, -0.5, 9.5));
  VMesons[0]->GetYaxis()->SetTitle("#frac{<n>}{<n_{ch}>}");
  VMesons[0]->GetXaxis()->SetBinLabel(1,"#pi^{#pm}");
  VMesons[0]->GetXaxis()->SetBinLabel(2,"#pi^{0}");
  VMesons[0]->GetXaxis()->SetBinLabel(3,"K^{#pm}");
  VMesons[0]->GetXaxis()->SetBinLabel(4,"#eta");
  VMesons[0]->GetXaxis()->SetBinLabel(5,"#eta'");
  VMesons[0]->GetXaxis()->SetBinLabel(6,"#rho^{#pm}");
  VMesons[0]->GetXaxis()->SetBinLabel(7,"#rho^{0}"); 
  VMesons[0]->GetXaxis()->SetBinLabel(8,"K^{*#pm}");
  VMesons[0]->GetXaxis()->SetBinLabel(9,"#omega");
  VMesons[0]->GetXaxis()->SetBinLabel(10,"#phi");
  VMesons[0]->GetXaxis()->SetLabelOffset(0.04);
  VMesons[0]->GetXaxis()->SetLabelSize(0.07);
  VMesons[0]->SetMaximum(50.01); 
  VMesons[0]->SetMinimum(0.0005); 
  if (hadronLevel) {
    vroot.createDisplay("Mesons",VMesons,plotSizeX,plotSizeY);
    vroot.setNormalize("Mesons",false);
    //vroot.setRatioRange("Mesons",0.5,1.5);
    vroot.setDataFile("Mesons","data/LEP91-mesons.vec",20,40,50);  
    for (int iRead=0; iRead<int(mcFileRead.size()); ++iRead) 
      vroot.addHisFile("Mesons",mcFileRead[iRead]+"-Mesons.dat");
  }

  // Baryons
  nBins = 11;
  xLo   = -0.5;
  xHi   = 10.5;
  vector<TH1D*> VBaryons;
  VBaryons.push_back(new TH1D("Baryons","Baryon Fractions",nBins, xLo, xHi));  
  VBaryons[0]->GetYaxis()->SetTitle("#frac{<n>}{<n_{ch}>}");
  VBaryons[0]->GetXaxis()->SetBinLabel(1,"p");
  VBaryons[0]->GetXaxis()->SetBinLabel(2,"#Lambda");
  VBaryons[0]->GetXaxis()->SetBinLabel(3,"#Lambda/p");
  VBaryons[0]->GetXaxis()->SetBinLabel(4,"#Lambda/K");
  VBaryons[0]->GetXaxis()->SetBinLabel(5,"#Sigma^{#pm}");
  VBaryons[0]->GetXaxis()->SetBinLabel(6,"#Sigma^{0}");
  VBaryons[0]->GetXaxis()->SetBinLabel(7,"#Delta");
  VBaryons[0]->GetXaxis()->SetBinLabel(8,"#Sigma^{*}");
  VBaryons[0]->GetXaxis()->SetBinLabel(9,"#Xi^{#pm}");
  VBaryons[0]->GetXaxis()->SetBinLabel(10,"#Xi^{*0}");
  VBaryons[0]->GetXaxis()->SetBinLabel(11,"#Omega");
  VBaryons[0]->GetXaxis()->SetLabelOffset(0.04);
  VBaryons[0]->GetXaxis()->SetLabelSize(0.07);
  VBaryons[0]->SetMaximum(5.01); 
  VBaryons[0]->SetMinimum(0.00002); 
  if (hadronLevel) {
    vroot.createDisplay("Baryons",VBaryons,plotSizeX,plotSizeY);
    vroot.setNormalize("Baryons",false);
    //vroot.setRatioRange("Baryons",0.5,1.5);
    vroot.setDataFile("Baryons","data/LEP91-baryons.vec",20,40,50);
    for (int iRead=0; iRead<int(mcFileRead.size()); ++iRead) 
      vroot.addHisFile("Baryons",mcFileRead[iRead]+"-Baryons.dat");
  }    

  // Heavy Mesons
  nBins =  12;
  xLo   = -0.5;
  xHi   = 11.5;
  vector<TH1D*> VMesonsHvy;
  VMesonsHvy.push_back(new TH1D("MesonsHvy","Heavy Meson Rates",nBins, xLo, xHi));
  VMesonsHvy[0]->GetYaxis()->SetTitle("<n>");
  VMesonsHvy[0]->GetXaxis()->SetBinLabel(1,"D^{#pm}");
  VMesonsHvy[0]->GetXaxis()->SetBinLabel(2,"D^{0}");
  VMesonsHvy[0]->GetXaxis()->SetBinLabel(3,"D^{*#pm}");
  VMesonsHvy[0]->GetXaxis()->SetBinLabel(4,"D_{s}^{#pm}");
  VMesonsHvy[0]->GetXaxis()->SetBinLabel(5,"B^{#pm}");
  VMesonsHvy[0]->GetXaxis()->SetBinLabel(6,"B^{#pm0}");
  VMesonsHvy[0]->GetXaxis()->SetBinLabel(7,"B^{*}_{uds}");
  VMesonsHvy[0]->GetXaxis()->SetBinLabel(8,"B_{s}^{0}");
  VMesonsHvy[0]->GetXaxis()->SetBinLabel(9,"J/#psi");
  VMesonsHvy[0]->GetXaxis()->SetBinLabel(10,"#chi_{c1}");
  VMesonsHvy[0]->GetXaxis()->SetBinLabel(11,"#psi_{3685}");
  VMesonsHvy[0]->GetXaxis()->SetBinLabel(12,"#Upsilon");
  VMesonsHvy[0]->GetXaxis()->SetLabelOffset(0.04);
  VMesonsHvy[0]->GetXaxis()->SetLabelSize(0.07);
  VMesonsHvy[0]->SetMaximum(20.0); 
  VMesonsHvy[0]->SetMinimum(0.00005); 
  if (hadronLevel) {
    vroot.createDisplay("MesonsHvy",VMesonsHvy,plotSizeX,plotSizeY);
    vroot.setNormalize("MesonsHvy",false);
    //vroot.setRatioRange("MesonsHvy",0.5,1.5);
    vroot.setDataFile("MesonsHvy","data/LEP91-heavymesons.vec",20,40,50,60);  
    for (int iRead=0; iRead<int(mcFileRead.size()); ++iRead) 
      vroot.addHisFile("MesonsHvy",mcFileRead[iRead]+"-MesonsHvy.dat");
  }

  // nCharged (udsc)
  nBins = 61;
  xLo   = -0.5;
  xHi   = 60.5;
  vector<TH1D*> VnCharged;
  VnCharged.push_back(new TH1D("nCh","Charged Multiplicity (udsc)",nBins,xLo,xHi));
  VnCharged[0]->GetXaxis()->SetTitle("n_{ch}");
  VnCharged[0]->GetYaxis()->SetTitle("Probability(n_{ch})");
  VnCharged[0]->SetMinimum(0.000005);
  VnCharged[0]->SetMaximum(5.);
  for (int iVar=1; iVar<nVariations; iVar++) 
    VnCharged.push_back(new TH1D(string("nCh"+num2str(iVar,0)).c_str(),
			       " ",nBins,xLo,xHi));
  if (hadronLevel) {
    vroot.createDisplay("Nch",VnCharged,plotSizeX,plotSizeY);
    vroot.setDataFile("Nch","data/L3-91-Nch-udsc.vec",20,20,40,51,52,61,62);
    for (int iRead=0; iRead<int(mcFileRead.size()); ++iRead) 
      vroot.addHisFile("Nch",mcFileRead[iRead]+"-Nch.dat");
    vroot.setLogY("Nch");
  }

  // Ln(1/x) (udsc)
  nBins = 45;
  xLo   = -0.2;
  xHi   = 8.8;
  vector<TH1D*> Vlnx;
  Vlnx.push_back(new TH1D("LnXInv","Charged Momentum Fraction (udsc)",nBins,xLo,xHi));
  double wLnx = nBins/(xHi-xLo);
  Vlnx[0]->GetXaxis()->SetTitle("|Ln(x_{p})|");
  Vlnx[0]->GetYaxis()->SetTitle("1/<n_{ch}> dn_{ch}/d|Ln(x_{p})|");
  Vlnx[0]->SetMinimum(0.0004);
  Vlnx[0]->SetMaximum(40.);
  for (int iVar=1; iVar<nVariations; iVar++) 
    Vlnx.push_back(new TH1D(string("LnXInv"+num2str(iVar,0)).c_str(),
			  " ",nBins,xLo,xHi));
  if (hadronLevel) {
    vroot.createDisplay("Lnx",Vlnx,plotSizeX,plotSizeY);
    vroot.setDataFile("Lnx","data/L3-91-LogInvx-udsc.vec",10,30,40,51,52,61,62);
    for (int iRead=0; iRead<int(mcFileRead.size()); ++iRead) 
      vroot.addHisFile("Lnx",mcFileRead[iRead]+"-Lnx.dat");
    vroot.setLogY("Lnx");
  }

  // x (all)
  nBins = 200;
  xLo   = 0.0;
  xHi   = 1.0;
  vector<TH1D*> Vx;
  Vx.push_back(new TH1D("X","Charged Momentum Fraction",nBins,xLo,xHi));
  double wx = nBins/(xHi-xLo);
  Vx[0]->GetXaxis()->SetTitle("x_{p}");
  Vx[0]->GetYaxis()->SetTitle("1/<n_{ch}> dn_{ch}/dx_{p}");
  Vx[0]->SetMinimum(0.00005);
  Vx[0]->SetMaximum(40.);
  for (int iVar=1; iVar<nVariations; iVar++) 
    Vx.push_back(new TH1D(string("LnXInv"+num2str(iVar,0)).c_str(),
			  " ",nBins,xLo,xHi));
  if (hadronLevel) {
    vroot.createDisplay("x",Vx,plotSizeX,plotSizeY);
    vroot.setDataFile("x","data/Aleph-91-x.vec",10,30,40,50,60);
    for (int iRead=0; iRead<int(mcFileRead.size()); ++iRead) 
      vroot.addHisFile("x",mcFileRead[iRead]+"-x.dat");
    vroot.setLogY("x");
  }

  // Rapidity wrt Thrust
  nBins = 24;
  xLo   = 0.0;
  xHi   = 6.0;
  vector<TH1D*> VyThr;
  VyThr.push_back(new TH1D("yThr","Charged Rapidity wrt Thrust",nBins,xLo,xHi));
  double wyThr = nBins/(xHi-xLo);
  VyThr[0]->GetXaxis()->SetTitle("|y|");
  VyThr[0]->GetYaxis()->SetTitle("1/<n_{ch}> dn_{ch}/d|y|");
  VyThr[0]->SetMinimum(0.00005);
  VyThr[0]->SetMaximum(20.);
  for (int iVar=1; iVar<nVariations; iVar++) 
    VyThr.push_back(new TH1D(string("yThr"+num2str(iVar,0)).c_str(),
			  " ",nBins,xLo,xHi));
  vroot.createDisplay("yThr",VyThr,plotSizeX,plotSizeY);
  vroot.setDataFile("yThr","data/Aleph-91-yThr.vec",10,30,40,51,52,61,62);
  for (int iRead=0; iRead<int(mcFileRead.size()); ++iRead) 
    vroot.addHisFile("yThr",mcFileRead[iRead]+"-yThr.dat");
  //vroot.setRatioRange("yThr",0.5,1.5);

  // x(K+-) from ALEPH, used to zoom in on the soft part of the spectrum
  nBins = 210;  // (needed due to non-uniform bins in data)
  xLo   = -5.25;
  xHi   = 0.;
  vector<TH1D*> VxKch;
  VxKch.push_back(new TH1D("xKch","x(K^{#pm}) w fiducial cuts",nBins,xLo,xHi));
  double wxKch = nBins/(xHi-xLo);
  VxKch[0]->GetXaxis()->SetTitle("x_{p}");
  VxKch[0]->GetYaxis()->SetTitle("1/<n_{K}> dn_{K}/dx");
  VxKch[0]->SetMinimum(0.0005);
  VxKch[0]->SetMaximum(50.);  
  for (int iVar=1; iVar<nVariations; iVar++) 
    VxKch.push_back(new TH1D(string("xKch"+num2str(iVar,0)).c_str(),
			       " ",nBins,xLo,xHi));
  if (hadronLevel && OPTshowParticleSpectra) {
    // Make this plot extra wide to show tiny bins near zero
    vroot.createDisplay("xKch",VxKch,plotSizeX,plotSizeY);
    vroot.setDataFile("xKch","data/Aleph-Opal-Delphi-91-xK.vec",13,33,43,50,60,71,72);
    for (int iRead=0; iRead<int(mcFileRead.size()); ++iRead) 
      vroot.addHisFile("xKch",mcFileRead[iRead]+"-xKch.dat");
    //vroot.setRatioRange("xKch",0.5,1.5);
  }

  // x(protons) ALEPH + SLD combined
  nBins = 253;  // (needed due to non-uniform bins in data)
  xLo   = -5;
  xHi   = 0.0;
  vector<TH1D*> Vxp;
  Vxp.push_back(new TH1D("xp","x(p+#bar{p}) w fiducial cuts",nBins,xLo,xHi));
  double wxp = nBins/(xHi-xLo);
  Vxp[0]->GetXaxis()->SetTitle("x_{p}");
  Vxp[0]->GetYaxis()->SetTitle("1/<n_{p}> dn_{p}/dx");
  Vxp[0]->SetMinimum(0.0005);
  Vxp[0]->SetMaximum(50.);
  for (int iVar=1; iVar<nVariations; iVar++) 
    Vxp.push_back(new TH1D(string("xp"+num2str(iVar,0)).c_str(),
			       " ",nBins,xLo,xHi));
  if (hadronLevel && OPTshowParticleSpectra) {
    vroot.createDisplay("xProton",Vxp,plotSizeX,plotSizeY);
    vroot.setDataFile("xProton","data/ALEPH-SLD-91-xp.vec",13,33,43,50,60,
                      71,72);
    for (int iRead=0; iRead<int(mcFileRead.size()); ++iRead) 
      vroot.addHisFile("xProton",mcFileRead[iRead]+"-xProton.dat");
    //vroot.setRatioRange("xProton",0.5,1.5);
  }

  // Lnx(Lambda)
  nBins = 25;  // (needed due to non-uniform bins in data)
  xLo   = 0;
  xHi   = 5.0;
  vector<TH1D*> VxLam;
  VxLam.push_back(new TH1D("xLam","|Ln[x(#Lambda^{0})]|",nBins,xLo,xHi));
  double wxLam = nBins/(xHi-xLo);
  VxLam[0]->GetXaxis()->SetTitle("#xi_{p}");
  VxLam[0]->GetYaxis()->SetTitle("1/<n_{#Lambda}> dn_{#Lambda}/d#xi");
  VxLam[0]->SetMinimum(0.007);
  VxLam[0]->SetMaximum(5.);
  for (int iVar=1; iVar<nVariations; iVar++) 
    VxLam.push_back(new TH1D(string("xLam"+num2str(iVar,0)).c_str(),
			       " ",nBins,xLo,xHi));
  if (hadronLevel && OPTshowParticleSpectra) {
    vroot.createDisplay("xLambda",VxLam,plotSizeX,plotSizeY);
    vroot.setDataFile("xLambda","data/Aleph-91-LogInvxL.vec",10,30,40,50,60);
    for (int iRead=0; iRead<int(mcFileRead.size()); ++iRead) 
      vroot.addHisFile("xLambda",mcFileRead[iRead]+"-xLambda.dat");
    //vroot.setRatioRange("xLambda",0.5,1.5);
  }

  // x(Xi)
  nBins = 55;  // (needed due to non-uniform bins in data)
  xLo   = 0;
  xHi   = 0.55;
  vector<TH1D*> VxXi;
  VxXi.push_back(new TH1D("xXi","x(#Xi^{#pm})",nBins,xLo,xHi));
  double wxXi = nBins/(xHi-xLo);
  VxXi[0]->GetXaxis()->SetTitle("x_{E}");
  VxXi[0]->GetYaxis()->SetTitle("1/<n_{#Xi}> dn_{#Xi}/dx");
  VxXi[0]->SetMinimum(0.07);
  VxXi[0]->SetMaximum(50.);
  for (int iVar=1; iVar<nVariations; iVar++) 
    VxXi.push_back(new TH1D(string("xXi"+num2str(iVar,0)).c_str(),
			       " ",nBins,xLo,xHi));
  if (hadronLevel && OPTshowParticleSpectra) {
    vroot.createDisplay("xXi",VxXi,plotSizeX,plotSizeY);
    vroot.setDataFile("xXi","data/Aleph-91-xXi.vec",10,30,40,50,0);
    vroot.setLogY("xXi");
    for (int iRead=0; iRead<int(mcFileRead.size()); ++iRead) 
      vroot.addHisFile("xXi",mcFileRead[iRead]+"-xXi.dat");
    //vroot.setRatioRange("xXi",0.5,1.5);
  }

  // PTIN vs Thrust Axes
  nBins = 140; // Important = multiplum of # of data bins, for ratio rebinning
  xLo =  0.0;
  xHi = 14.0;
  vector<TH1D*> VPTinThr;
  VPTinThr.push_back(new TH1D("PTinThr","Charged p_{T}^{in} (Thrust Axes)",
                              nBins,xLo,xHi));
  double wPTinThr = nBins/(xHi-xLo);
  VPTinThr[0]->GetXaxis()->SetTitle("p_{T}^{in}");
  VPTinThr[0]->GetYaxis()->SetTitle("1/<n_{ch}> dn_{ch}/dp_{T}");
  VPTinThr[0]->SetMinimum(0.000005);
  VPTinThr[0]->SetMaximum(11.);
  for (int iVar=1; iVar<nVariations; iVar++) 
    VPTinThr.push_back(new TH1D(string("PTinThr"+num2str(iVar,0)).c_str(),
                                " ",nBins,xLo,xHi));  
  if (hadronLevel && OPTshowPT) {
    vroot.createDisplay("PTinThr",VPTinThr,plotSizeX,plotSizeY);
    vroot.setDataFile("PTinThr","data/Delphi-91-PTIN.vec",10,30,40,51,52,61,62);
    for (int iRead=0; iRead<int(mcFileRead.size()); ++iRead) 
      vroot.addHisFile("PTinThr",mcFileRead[iRead]+"-PTinThr.dat");
    vroot.setLogY("PTinThr");
  }

  // PTIN vs Sphericity Axes
  nBins = 140; // Important = multiplum of # of data bins, for ratio rebinning
  xLo =  0.0;
  xHi = 14.0;
  vector<TH1D*> VPTinSph;
  VPTinSph.push_back(new TH1D("PTinSph","Charged p_{T}^{in} (Sphericity Axes)",
                              nBins,xLo,xHi));
  double wPTinSph = nBins/(xHi-xLo);
  VPTinSph[0]->GetXaxis()->SetTitle("p_{T}^{in}");
  VPTinSph[0]->GetYaxis()->SetTitle("1/<n_{ch}> dn_{ch}/dp_{T}");
  VPTinSph[0]->SetMinimum(0.000005);
  VPTinSph[0]->SetMaximum(11.);
  for (int iVar=1; iVar<nVariations; iVar++) 
    VPTinSph.push_back(new TH1D(string("PTinSph"+num2str(iVar,0)).c_str(),
                                " ",nBins,xLo,xHi));  
  if (hadronLevel && OPTshowPT) {
    vroot.createDisplay("PTinSph",VPTinSph,plotSizeX,plotSizeY);
    vroot.setDataFile("PTinSph","data/Aleph-91-PTIN-s.vec",
                      10,30,40,51,52,61,62);
    for (int iRead=0; iRead<int(mcFileRead.size()); ++iRead) 
      vroot.addHisFile("PTinSph",mcFileRead[iRead]+"-PTinSph.dat");
    vroot.setLogY("PTinSph");
  }

  // PTOUT vs Thrust Axes
  nBins = 50; // Important = multiplum of # of data bins, for ratio rebinning
  xLo =  0.0;
  xHi =  5.0;
  vector<TH1D*> VPToutThr;
  VPToutThr.push_back(new TH1D("PToutThr","Charged p_{T}^{out} (Thrust Axes)",
                               nBins,xLo,xHi));
  double wPToutThr = nBins/(xHi-xLo);
  VPToutThr[0]->GetXaxis()->SetTitle("p_{T}^{out}");
  VPToutThr[0]->GetYaxis()->SetTitle("1/<n_{ch}> dn_{ch}/dp_{T}");
  VPToutThr[0]->SetMinimum(0.000005);
  VPToutThr[0]->SetMaximum(11.);
  for (int iVar=1; iVar<nVariations; iVar++) 
    VPToutThr.push_back(new TH1D(string("PToutThr"+num2str(iVar,0)).c_str(),
                                 " ",nBins,xLo,xHi));  
  if (hadronLevel && OPTshowPT) {
    vroot.createDisplay("PToutThr",VPToutThr,plotSizeX,plotSizeY);
    vroot.setDataFile("PToutThr","data/Delphi-91-PTOUT.vec",
                      10,30,40,51,52,61,62);
    for (int iRead=0; iRead<int(mcFileRead.size()); ++iRead) 
      vroot.addHisFile("PToutThr",mcFileRead[iRead]+"-PToutThr.dat");
    vroot.setLogY("PToutThr");
  }

  // PTIN vs Sphericity Axes
  nBins = 50; // Important = multiplum of # of data bins, for ratio rebinning
  xLo =  0.0;
  xHi =  5.0;
  vector<TH1D*> VPToutSph;
  VPToutSph.push_back(new TH1D("PToutSph",
                               "Charged p_{T}^{out} (Sphericity Axes)",
                               nBins,xLo,xHi));
  double wPToutSph = nBins/(xHi-xLo);
  VPToutSph[0]->GetXaxis()->SetTitle("p_{T}^{out}");
  VPToutSph[0]->GetYaxis()->SetTitle("1/<n_{ch}> dn_{ch}/dp_{T}");
  VPToutSph[0]->SetMinimum(0.000005);
  VPToutSph[0]->SetMaximum(11.);
  for (int iVar=1; iVar<nVariations; iVar++) 
    VPToutSph.push_back(new TH1D(string("PToutSph"+num2str(iVar,0)).c_str(),
                                 " ",nBins,xLo,xHi));  
  if (hadronLevel && OPTshowPT) {
    vroot.createDisplay("PToutSph",VPToutSph,plotSizeX,plotSizeY);
    vroot.setDataFile("PToutSph","data/Aleph-91-PTOUT-s.vec",
                      10,30,40,51,52,61,62);
    for (int iRead=0; iRead<int(mcFileRead.size()); ++iRead) 
      vroot.addHisFile("PToutSph",mcFileRead[iRead]+"-PToutSph.dat");
    vroot.setLogY("PToutSph");
  }

  // Thrust (non-B flavor)
  nBins = 18; // Important = multiplum of # of data bins, for ratio rebinning
  xLo = 0.0;
  xHi = 0.45;
  vector<TH1D*> VThrustLight;
  VThrustLight.push_back(new TH1D("ThrustLight","1-Thrust (udsc)",nBins,xLo,xHi));
  double wThrust = nBins/(xHi-xLo);
  VThrustLight[0]->GetXaxis()->SetTitle("1-T (udsc)");
  VThrustLight[0]->GetYaxis()->SetTitle("1/#sigma d#sigma/d(1-T)");
  VThrustLight[0]->SetMinimum(0.0005);
  VThrustLight[0]->SetMaximum(200.);
  for (int iVar=1; iVar<nVariations; iVar++) 
    VThrustLight.push_back(new TH1D(string("ThrustLight"+num2str(iVar,0)).c_str(),
				    " ",nBins,xLo,xHi));  
  vroot.createDisplay("T",VThrustLight,plotSizeX,plotSizeY);
  vroot.setDataFile("T","data/L3-91-Thrust-udsc.vec",10,30,40,51,52,61,62);
  for (int iRead=0; iRead<int(mcFileRead.size()); ++iRead) 
    vroot.addHisFile("T",mcFileRead[iRead]+"-T.dat");
  vroot.setLogY("T");

  // C (light)
  nBins = 20; // Important = multiplum of # of data bins, for ratio rebinning
  xLo   = 0.0;
  xHi   = 1.0;
  vector<TH1D*> VCLight;
  VCLight.push_back(new TH1D("CLight","C Parameter (udsc)",nBins,xLo,xHi));
  double wC = nBins/(xHi-xLo);
  VCLight[0]->GetXaxis()->SetTitle("C");
  VCLight[0]->GetYaxis()->SetTitle("1/#sigma d#sigma/dC");
  VCLight[0]->SetMinimum(0.0005);
  VCLight[0]->SetMaximum(200.);
  for (int iVar=1; iVar<nVariations; iVar++) 
    VCLight.push_back(new TH1D(string("CLight"+num2str(iVar,0)).c_str(),
			       " ",nBins,xLo,xHi));
  vroot.createDisplay("C",VCLight,plotSizeX,plotSizeY);
  vroot.setDataFile("C","data/L3-91-C-udsc.vec",10,30,40,51,52,61,62);
  for (int iRead=0; iRead<int(mcFileRead.size()); ++iRead) 
    vroot.addHisFile("C",mcFileRead[iRead]+"-C.dat");
  vroot.setLogY("C");

  // D (udsc)
  nBins = 25; // Important = multiplum of # of data bins, for ratio rebinning
  xLo   = 0.0;
  xHi   = 0.8;
  vector<TH1D*> VDLight;
  VDLight.push_back(new TH1D("DLight","D Parameter (udsc)",nBins,xLo,xHi));
  double wD = nBins*1.0/(xHi-xLo);
  VDLight[0]->GetXaxis()->SetTitle("D");
  VDLight[0]->GetYaxis()->SetTitle("1/#sigma d#sigma/dD");
  VDLight[0]->SetMinimum(0.0005);
  VDLight[0]->SetMaximum(50.);
  for (int iVar=1; iVar<nVariations; iVar++) 
    VDLight.push_back(new TH1D(string("DLight"+num2str(iVar,0)).c_str(),
			       " ",nBins,xLo,xHi));
  vroot.createDisplay("D",VDLight,plotSizeX,plotSizeY);
  vroot.setDataFile("D","data/L3-91-D-udsc.vec",10,30,40,51,52,61,62);
  for (int iRead=0; iRead<int(mcFileRead.size()); ++iRead) 
    vroot.addHisFile("D",mcFileRead[iRead]+"-D.dat");
  vroot.setLogY("D");

  // Wide Jet Broadening
  nBins = 20; // Important = multiplum of # bins in data, for ratio rebinning
  xLo = 0.0;
  xHi = 0.3;
  vector<TH1D*> VBjWide;
  VBjWide.push_back(new TH1D("BWide","Wide Jet Broadening (udsc)",nBins,xLo,xHi));
  double wBjWide = nBins/(xHi-xLo);
  VBjWide[0]->GetXaxis()->SetTitle("B_{W}");
  VBjWide[0]->GetYaxis()->SetTitle("1/#sigma d#sigma/dB_{W}");
  VBjWide[0]->SetMaximum(500.0); 
  VBjWide[0]->SetMinimum(0.005);   
  for (int iVar=1; iVar<nVariations; iVar++) 
    VBjWide.push_back(new TH1D(string("BWide"+num2str(iVar,0)).c_str(),
			       " ",nBins,xLo,xHi));
  vroot.createDisplay("BW",VBjWide,plotSizeX,plotSizeY);
  vroot.setDataFile("BW","data/L3-91-BWide-udsc.vec",20,10,30,40,51,52,61,62);
  for (int iRead=0; iRead<int(mcFileRead.size()); ++iRead) 
    vroot.addHisFile("BW",mcFileRead[iRead]+"-BW.dat");
  vroot.setLogY("BW");

  // Total Jet Broadening
  nBins = 20; // Must match or be multiplum of # bins in data, for ratio rebinning
  xLo = 0.0;
  xHi = 0.4;
  vector<TH1D*> VBjTot;
  VBjTot.push_back(new TH1D("BTot","Total Jet Broadening (udsc)",
                            nBins, xLo, xHi));
  double wBjTot = 200.0/0.4;
  VBjTot[0]->GetXaxis()->SetTitle("B_{T}");
  VBjTot[0]->GetYaxis()->SetTitle("1/#sigma d#sigma/dB_{T}");
  VBjTot[0]->SetMaximum(500.0); 
  VBjTot[0]->SetMinimum(0.00101);     
  for (int iVar=1; iVar<nVariations; iVar++) 
    VBjTot.push_back(new TH1D(string("BTot"+num2str(iVar,0)).c_str(),
			       " ",nBins,xLo,xHi));
  vroot.createDisplay("BT",VBjTot,plotSizeX,plotSizeY);
  vroot.setDataFile("BT","data/L3-91-BTot-udsc.vec",20,10,30,40,51,52,61,62);
  for (int iRead=0; iRead<int(mcFileRead.size()); ++iRead) 
    vroot.addHisFile("BT",mcFileRead[iRead]+"-BT.dat");
  vroot.setLogY("BT");

  // Event analysis tools
  Thrust Thrust(1), ThrustCh(3);
  Sphericity SphLin(1,1),SphSqCh(2,3);;

  //***************************************************************************
  // EVENT GENERATION LOOP
  // Generation, event-by-event printout, analysis, and storage
  //***************************************************************************

  // Number of quarks, partons, and charged particles
  double nQsum=0.0;
  double nPsum=0.0;
  double nCsum=0.0;
  double nMesSum[20] = {0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.};
  double nBarSum[12] = {0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.};
  double nMesHvySum[12] = {0., 0., 0., 0., 0., 0., 0., 0., 0., 0. ,0., 0.};
  double nEvDch = 0.0;
  double nEvD0 = 0.0;

  // Begin event loop
  for (int iEvent = 0; iEvent < nEvent; ++iEvent) {

    // Generate next event
    if (!pythia.next()) continue;
    event = &pythia.event;

    // Count FS charged hadrons, partons, and quarks
    int nc=0;
    int np=0;
    int nq=0;

    // Bool to reject events with B's
    bool isB = false;
    bool isDch = false;
    bool isD0 = false;
    for (int i=1;i<event->size();i++) 
      if (abs((*event)[i].id()) == 5) isB = true;  
 
    // Main loop over event record
    for (int i=1;i<event->size();i++) {

      // Count up final-state charged hadrons
      double x = 2.0*(*event)[i].pAbs()/eCM;
      double xE = 2.0*(*event)[i].e()/eCM;
      double xi = -log(x);
      if ((*event)[i].isCharged() && (*event)[i].status() > 80) {
	nc++;
        for (int iVar=0; iVar<nVariations; iVar++) {
          if (vincia.weight(iVar) > 0.0) {
            if (! isB) Vlnx[iVar]->Fill(xi,wLnx*vincia.weight(iVar));
            if (x>0.02) Vx[iVar]->Fill(x,wx*vincia.weight(iVar));
          }
        }
      }
      // Find last parton-level partons
      int iDau1 = (*event)[i].daughter1();
      if (iDau1 == 0 || abs((*event)[iDau1].status()) > 80) {
	// Count up partons and quarks
	if ((*event)[i].isQuark() || (*event)[i].isGluon() ) np++;
	if ((*event)[i].isQuark()) nq++;	
      }
      // Find identified particles
      /* Mesons:
	# 211 0: e+ e- --> pi+- X
	# 111 1: e+ e- --> pi0 X
	# 321 2: e+ e- --> K+- X
	# 221 3: e+ e- --> eta X
	# 331 4: e+ e- --> etaprime(958) X
	# 213 5: e+ e- --> rho(770)+- X
	# 113 6: e+ e- --> rho(770)0 X
	# 323 7: e+ e- --> K*(892)+- X
	# 223 8: e+ e- --> omega(782) X
	# 333 9: e+ e- --> phi(1020) X
	# 10111 10: e+ e- --> a0(980) X
	# 10221 11: e+ e- --> f0(980) X
      */
      /* Baryons:
	 # 2212      0 : e+ e- --> p (pbar) X
	 # 3122      1: e+ e- --> Lambda (Lambdabar) X
	 # 3122/2212 2: e+ e- --> Lambda (Lambdabar) / p(pbar)
	 # 3122/321  3: e+ e- --> Lambda (Lambdabar) / K+-
	 # 3222+3112 4: e+ e- --> Sigma+- (Sigmabar+-) X
	 # 3212      5: e+ e- --> Sigma0 (Sigmabar0) X
	 # 2224      6 : e+ e- --> Delta(1232)++ (Delta(1232)--) X
	 # 3224+3114 7: e+ e- --> Sigma(1385)+- (Sigmabar(1385)+-) X
	 # 3312      8: e+ e- --> Xi- (Xibar+) X
	 # 3324      9: e+ e- --> Xi(1530)0 (Xibar(1530)0) X
	 # 3334     10: e+ e- --> Omega- (Omegabar+) X 
      */
      /* Heavy Mesons:
	 # 411    0:  e+ e- --> D+- X (A)
         # 421    1: e+ e- --> D0 (Dbar0) X (A)
         # 413    2: e+ e- --> D*(2010)+- X (O)
	 # 431    3: e+ e- --> D+-s X (PDG)
         # 443    4: e+ e- --> J/psi X (A)
         # 443    5: e+ e- --> J/psi X (D)
         # 443    6: e+ e- --> J/psi X (L)
         # 443    7: e+ e- --> J/psi X (O)
         # 20443  8: e+ e- --> chi(c1) X (L)
         # 100443 9: e+ e- --> psi(3685) X (L)	
      */
      int idp = abs((*event)[i].id());
      if (idp == 211) nMesSum[0]++;
      if (idp == 111) nMesSum[1]++;
      if (idp == 321) {
	nMesSum[2]++;
        // SLD charged kaons
        /*
	if ( (x >= 0.016 && x <= 0.164) 
	     || (x >= 0.208 && x <= 0.768) ) {
	  for (int iVar=0; iVar<nVariations; iVar++) 
	    VxKchSLD[iVar]->Fill(x,wxKchSLD*vincia.weight(iVar)); 
	}
        */
        // ALEPH, DELPHI, OPAL charged kaons
	if ( (xi <= 5.2 && xi >= 4.0) || (xi <= 3.925) ) {
	  for (int iVar=0; iVar<nVariations; iVar++) {
	    VxKch[iVar]->Fill(-xi,wxKch*vincia.weight(iVar));         
          }
        }
      }
      if (idp == 221) nMesSum[3]++;
      if (idp == 331) nMesSum[4]++;
      if (idp == 213) nMesSum[5]++;
      if (idp == 113) nMesSum[6]++;
      if (idp == 323) nMesSum[7]++;
      if (idp == 223) nMesSum[8]++;
      if (idp == 333) nMesSum[9]++;
      if (idp == 10111) nMesSum[10]++;
      if (idp == 10221) nMesSum[11]++;
      if (idp == 2212) {
	nBarSum[0]++;
	// ALEPH + SLD combined protons
	if ( (xi <= 4.6047 && xi>=4.012 ) 
	     || (xi <= 3.735 && xi >= 3.578) 
	     || (xi <= 3.4189 && xi >= 1.799) 
	     || (xi <= 1.482) ) 
	  for (int iVar=0; iVar<nVariations; iVar++) {
	    Vxp[iVar]->Fill(-xi,vincia.weight(iVar)*wxp);
          }
      }
      if (idp == 3122) {
	nBarSum[1]++;
	if (xi >= 0.4 && xi <= 4.8)
	  for (int iVar=0; iVar<nVariations; iVar++) {
	    VxLam[iVar]->Fill(xi,vincia.weight(iVar)*wxLam);
          }
      }
      if (idp == 3222) nBarSum[4]++; 
      if (idp == 3112) nBarSum[4]++; 
      if (idp == 3212) nBarSum[5]++; 
      if (idp == 2224) nBarSum[6]++; 
      if (idp == 3114) nBarSum[7]++; 
      if (idp == 3224) nBarSum[7]++; 
      if (idp == 3312) {
	nBarSum[8]++; 
	if (xE >= 0.03 && xE <= 0.5) 
	  for (int iVar=0; iVar<nVariations; iVar++) {
            // if (iVar != 0 && iVar != 9) continue;
	    VxXi[iVar]->Fill(xE,vincia.weight(iVar)*wxXi);
          }
      }
      if (idp == 3324) nBarSum[9]++; 
      if (idp == 3334) nBarSum[10]++; 

      // Heavy Mesons
      if (idp == 411) {
        isDch = true;
        nMesHvySum[0]++;
      }
      if (idp == 421) {
        isD0 = true;
        nMesHvySum[1]++;
      }
      if (idp == 413) nMesHvySum[2]++;
      if (idp == 431) nMesHvySum[3]++;
      if (idp == 443) nMesHvySum[8]++;
      if (idp == 20443) nMesHvySum[9]++;
      if (idp == 100443) nMesHvySum[10]++;
      if (idp == 511) nMesHvySum[4]++;
      if (idp == 511 || idp == 521) nMesHvySum[5] += 0.333;
      if (idp == 513 || idp == 523 || idp == 533) nMesHvySum[6]++;
      if (idp == 531) nMesHvySum[7]++;
      if (idp == 553) nMesHvySum[11]++;
    }

    // Fill Histograms
    nQsum += 0.5*nq;
    nPsum += np;
    nCsum += nc;
    nEvDch += (isDch ? 1 : 0);
    nEvD0 += (isD0 ? 1 : 0);
    HnPartons->Fill(1.0*np,1.);    
    for (int iVar=0; iVar<nVariations; iVar++) {
      if (vincia.weight(iVar) <= 0.0) continue;
      if (! isB) VnCharged[iVar]->Fill(1.0*nc,vincia.weight(iVar));
      VnQuarks[iVar]->Fill(0.5*nq,vincia.weight(iVar));
    }

    // Fill PID histograms
    if (nCsum > 0) {
      VMesons[0]->Reset();
      for (int iMes=0; iMes<=9; iMes++) {	
	VMesons[0]->Fill(double(iMes),nMesSum[iMes]/nCsum);
	double err = sqrt(nMesSum[iMes]) / nCsum;
	VMesons[0]->SetBinError(double(iMes)+1,err);
      }
      VBaryons[0]->Reset();
      for (int iBar=0; iBar<=10; iBar++) {
	VBaryons[0]->Fill(double(iBar),nBarSum[iBar]/nCsum);
	double err = sqrt(nBarSum[iBar]) / nCsum;
	VBaryons[0]->SetBinError(double(iBar)+1,err);
      }
      VMesonsHvy[0]->Reset();
      for (int iMes=0; iMes<=11; iMes++) {
	VMesonsHvy[0]->Fill(double(iMes),nMesHvySum[iMes]/(iEvent+1));
	double err = sqrt(nMesHvySum[iMes])/(iEvent+1);
	VMesonsHvy[0]->SetBinError(double(iMes)+1,err);
      }
      // Fill Lambda/p and Lambda/K
      VBaryons[0]->Fill(double(2),nBarSum[1]/nBarSum[0]);
      double err = sqrt(nBarSum[1]) / nBarSum[0];
      VBaryons[0]->SetBinError(double(2)+1,err);
      VBaryons[0]->Fill(double(3),nBarSum[1]/nMesSum[2]);
      err = sqrt(nBarSum[1]) / nMesSum[2];
      VBaryons[0]->SetBinError(double(3)+1,err);
    }

    // Thrust
    Thrust.analyze( *event );
    ThrustCh.analyze( *event );
    // Force light quarks only
    if (! isB) {
      for (int iVar=0; iVar<nVariations; iVar++) {
        VThrustLight[iVar]->Fill(1-Thrust.thrust(),wThrust*vincia.weight(iVar));
      }
    }
    
    // Histogram Sphericity values
    if (np >= 2.0) {
      SphLin.analyze( *event );
      SphSqCh.analyze( *event );
      double evC=3*(SphLin.eigenValue(1)*SphLin.eigenValue(2)
		    + SphLin.eigenValue(2)*SphLin.eigenValue(3)
		    + SphLin.eigenValue(3)*SphLin.eigenValue(1));
      double evD=27*SphLin.eigenValue(1)*SphLin.eigenValue(2)
	*SphLin.eigenValue(3);
      // Light quarks
      if (! isB) {
        for (int iVar=0; iVar<nVariations; iVar++) {
          VCLight[iVar]->Fill(evC,wC*vincia.weight(iVar));
          VDLight[iVar]->Fill(evD,wD*vincia.weight(iVar));
        }
      }

      // Histogram jet masses
      Vec4 thrustAxis = Thrust.eventAxis(1);   
      Vec4 majorChAxis  = ThrustCh.eventAxis(2);   
      Vec4 minorChAxis  = ThrustCh.eventAxis(3);   
      //Vec4 sphChAxis    = SphSqCh.eventAxis(1);   
      Vec4 sphChAxis2   = SphSqCh.eventAxis(2);   
      Vec4 sphChAxis3   = SphSqCh.eventAxis(3);   
      Vec4 pPlus, pMinus;
      double bPlus(0.), bMinus(0.), pPlusAbs(0.), pMinusAbs(0.);
      double pToutThr(0.), pTinThr(0.), pToutSph(0.), pTinSph(0.);
      // Loop over event to compute masses
      for (int i=0; i<event->size(); i++) {
        if (not (*event)[i].isFinal() ) continue;
        int id = (*event)[i].id();
        if (abs(id) == 12 || abs(id) == 14 || abs(id) == 16) continue;
        Particle& pNow = (*event)[i];
        // pTin and pTout
        if (pNow.isCharged() && OPTshowPT) {
          pTinThr  = abs(pNow.px()*majorChAxis.px() 
                     + pNow.py()*majorChAxis.py() + pNow.pz()*majorChAxis.pz());
          pToutThr = abs(pNow.px()*minorChAxis.px() 
                     + pNow.py()*minorChAxis.py() + pNow.pz()*minorChAxis.pz());
          pTinSph  = abs(pNow.px()*sphChAxis2.px() 
                     + pNow.py()*sphChAxis2.py() + pNow.pz()*sphChAxis2.pz());
          pToutSph = abs(pNow.px()*sphChAxis3.px() 
                     + pNow.py()*sphChAxis3.py() + pNow.pz()*sphChAxis3.pz());
          for (int iVar=0; iVar<nVariations; iVar++) {
            VPTinThr[iVar]->Fill(pTinThr,wPTinThr*vincia.weight(iVar));
            VPToutThr[iVar]->Fill(pToutThr,wPToutThr*vincia.weight(iVar));
            VPTinSph[iVar]->Fill(pTinSph,wPTinSph*vincia.weight(iVar));
            VPToutSph[iVar]->Fill(pToutSph,wPToutSph*vincia.weight(iVar));
          }          
        }
        // Jet masses and broadening: sums
        double dotp = pNow.px()*thrustAxis.px()
          + pNow.py()*thrustAxis.py() + pNow.pz()*thrustAxis.pz();
        double pAbs = pNow.pAbs();
        double cosT = dotp/pAbs/thrustAxis.pAbs();
        double sinT = sqrt(1.-pow2(cosT));
        if (dotp > 0) {
          pPlus     += pNow.p();
          bPlus     += abs(pAbs * sinT);
          pPlusAbs  += pAbs;
        } else {
          pMinus    += pNow.p();
          bMinus    += abs(pAbs * sinT);
          pMinusAbs += pAbs;
        }
        // Charged-particle Rapidity wrt Thrust axis
        if (pNow.isCharged() || pNow.isGluon()) {
          double pz = pNow.px()*thrustAxis.px() 
            + pNow.py()*thrustAxis.py() + pNow.pz()*thrustAxis.pz();
          double E  = pNow.e();
          double yThr = (E-pz != 0.) ? 0.5*log( (E+pz) / (E-pz) ) : 10.0;
          for (int iVar=0; iVar<nVariations; iVar++) {
            VyThr[iVar]->Fill(abs(yThr),wyThr*vincia.weight(iVar));          
          }
        }
      }
      bPlus  /= 2.*(pPlusAbs+pMinusAbs);
      bMinus /= 2.*(pMinusAbs+pPlusAbs);    
      if (! isB) {
        for (int iVar=0; iVar<nVariations; iVar++) {
          VBjWide[iVar]->Fill(max(bPlus,bMinus),
                              wBjWide*vincia.weight(iVar));
          VBjTot[iVar]->Fill(bPlus+bMinus,wBjTot*vincia.weight(iVar));
        }
      }
    }

    // Periodically update displays
    if ( iEvent == int(iSave) || (iEvent+251)%100 == 0 || iEvent == nEvent-1)
      vroot.updateDisplays();       

    // Periodically save displays
    if (iEvent == int(iSave) || iEvent == nEvent-1) {
      iSave *= sqrt(2.0);
      iSave = min(iSave,double(nEvent-1));
      cout<<"vincia03-root: saving plots at event "<<int(iEvent);
      cout<<" next save at event "<<int(iSave)<<endl;

      // Save displays with base name mcFile and file type .pdf 
      // (it is possible to choose other formats, like eps or png)
      vroot.saveDisplays(mcFileSave,"pdf");
      // Also save histograms as dat files
      vroot.saveHistograms(mcFileSave,"dat");
      // Print chi2 values
      // vroot.printChi2();
      // Print stats
      cout<<" <nCh>    = "<<nCsum/double(iEvent+1)<<endl;

    }
  }  

  // Print out Vincia internal histograms
  if (vinciaOn && verbose >= 2) vincia.printHistos();

  // Print out Vincia end-of-run information
  pythia.statistics();
  if (vinciaOn) vincia.printInfo();

  cout << endl << "Enter q to terminate (will kill displays)..." << endl;
  char tmp;cin>>tmp;
  
  return 0;
}
